package com.slibra.web.controller.business;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.slibra.common.annotation.Log;
import com.slibra.common.core.controller.BaseController;
import com.slibra.common.core.domain.AjaxResult;
import com.slibra.common.enums.BusinessType;
import com.slibra.business.domain.CBucket;
import com.slibra.business.service.ICBucketService;
import com.slibra.common.utils.poi.ExcelUtil;
import com.slibra.common.core.page.TableDataInfo;

/**
 * 流量分桶Controller
 * 
 * @author slibra
 * @date 2025-01-06
 */
@RestController
@RequestMapping("/business/bucket")
public class CBucketController extends BaseController
{
    @Autowired
    private ICBucketService cBucketService;

    /**
     * 查询流量分桶列表
     */
    @PreAuthorize("@ss.hasPermi('business:bucket:list')")
    @GetMapping("/list")
    public TableDataInfo list(CBucket cBucket)
    {
        startPage();
        List<CBucket> list = cBucketService.selectCBucketList(cBucket);
        return getDataTable(list);
    }

    /**
     * 导出流量分桶列表
     */
    @PreAuthorize("@ss.hasPermi('business:bucket:export')")
    @Log(title = "流量分桶", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CBucket cBucket)
    {
        List<CBucket> list = cBucketService.selectCBucketList(cBucket);
        ExcelUtil<CBucket> util = new ExcelUtil<CBucket>(CBucket.class);
        util.exportExcel(response, list, "流量分桶数据");
    }

    /**
     * 获取流量分桶详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:bucket:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return success(cBucketService.selectCBucketById(id));
    }

    /**
     * 新增流量分桶
     */
    @PreAuthorize("@ss.hasPermi('business:bucket:add')")
    @Log(title = "流量分桶", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CBucket cBucket)
    {
        cBucket.setCreateBy(getUsername());
        return toAjax(cBucketService.insertCBucket(cBucket));
    }

    /**
     * 修改流量分桶
     */
    @PreAuthorize("@ss.hasPermi('business:bucket:edit')")
    @Log(title = "流量分桶", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CBucket cBucket)
    {
        cBucket.setUpdateBy(getUsername());
        return toAjax(cBucketService.updateCBucket(cBucket));
    }

    /**
     * 删除流量分桶
     */
    @PreAuthorize("@ss.hasPermi('business:bucket:remove')")
    @Log(title = "流量分桶", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(cBucketService.deleteCBucketByIds(ids));
    }
}
