package com.slibra.web.controller.business;

import org.springframework.core.io.ByteArrayResource;
import org.springframework.core.io.InputStreamResource;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

@RestController
@RequestMapping("/api/files")
public class FileDownloadController {

    //目录
    private static final String FILE_STORAGE_LOCATION = "/Users/wangmiaomiao/Downloads/mlvrwj.20250217/";

    @GetMapping("/download")
    public ResponseEntity<InputStreamResource> downloadFile(@RequestParam String fileName) throws IOException {
        // 构造文件的完整路径
        File file = new File(FILE_STORAGE_LOCATION + fileName);

        // 检查文件是否存在
        if (!file.exists()) {
            throw new RuntimeException("File not found " + fileName);
        }

        // 读取文件内容到字节数组
        byte[] fileData = Files.readAllBytes(Paths.get(file.getAbsolutePath()));

        // 设置HTTP响应头
        HttpHeaders header = new HttpHeaders();
        header.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + file.getName() + "\"");
        header.add(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_OCTET_STREAM_VALUE);

        // 返回文件内容作为响应体
        return ResponseEntity.ok()
                .headers(header)
                .body(new InputStreamResource(new ByteArrayInputStream(fileData)));
    }


    @PostMapping("/downloadBatch")
    public ResponseEntity<Resource> downloadBatchFiles(@RequestBody List<String> fileNames) throws IOException {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        try (ZipOutputStream zipOutputStream = new ZipOutputStream(byteArrayOutputStream)) {
            for (String fileName : fileNames) {
                File file = new File(FILE_STORAGE_LOCATION + fileName);
                if (!file.exists()) {
                    throw new RuntimeException("File not found: " + fileName);
                }
                addFileToZip(file, file.getName(), zipOutputStream);
            }
        }

        byte[] zipContent = byteArrayOutputStream.toByteArray();
        ByteArrayResource resource = new ByteArrayResource(zipContent);

        HttpHeaders headers = new HttpHeaders();
        headers.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"batch_download.zip\"");
        headers.add(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_OCTET_STREAM_VALUE);

        return ResponseEntity.ok()
                .headers(headers)
                .contentLength(zipContent.length)
                .body(resource);
    }

    private void addFileToZip(File file, String entryName, ZipOutputStream zipOut) throws IOException {
        try (FileInputStream fis = new FileInputStream(file)) {
            ZipEntry zipEntry = new ZipEntry(entryName);
            zipOut.putNextEntry(zipEntry);

            byte[] bytes = new byte[1024];
            int length;
            while ((length = fis.read(bytes)) >= 0) {
                zipOut.write(bytes, 0, length);
            }
        }
    }

    // 互联网上文件的URL
    private static final String REMOTE_FILE_URL = "https://static.fuxicarbon.com/userupload/ttt.wav";

    // 缓存下载文件的临时目录（确保这个目录存在并且应用有写入权限）
    private static final String TEMP_DIR = "/Users/wangmiaomiao/Downloads/";

    // 缓存文件的文件名（可以从URL中解析出来，或者根据需求设置）
//    private static final String CACHED_FILE_NAME = "downloaded_file.zip";
    private static final String CACHED_FILE_NAME = "ttt.wav";

    @GetMapping("/downloadHttpUrl")
    public ResponseEntity<InputStreamResource> downloadFile() throws IOException {
        File cachedFile = new File(TEMP_DIR + CACHED_FILE_NAME);

        // 如果文件不存在，则从互联网上下载
        if (!cachedFile.exists()) {
            File tempDir = new File(TEMP_DIR);
            if (!tempDir.exists()) {
                tempDir.mkdirs();
            }
            downloadFileFromInternet(REMOTE_FILE_URL, cachedFile);
        }

        // 设置HTTP响应头
        HttpHeaders headers = new HttpHeaders();
        headers.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + CACHED_FILE_NAME + "\"");
        headers.add(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_OCTET_STREAM_VALUE);
        headers.setContentLength(cachedFile.length());

        // 返回文件内容作为响应体
        InputStream inputStream = Files.newInputStream(cachedFile.toPath());
        InputStreamResource resource = new InputStreamResource(inputStream);

        return ResponseEntity.ok()
                .headers(headers)
                .body(resource);
    }

    private void downloadFileFromInternet(String fileURL, File destination) throws IOException {
        URL url = new URL(fileURL);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestMethod("GET");

        try (InputStream inputStream = connection.getInputStream();
             FileOutputStream outputStream = new FileOutputStream(destination)) {

            byte[] buffer = new byte[4096];
            int bytesRead;
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
            }
        }
    }

}
