package com.slibra.web.controller.business;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.slibra.common.annotation.Log;
import com.slibra.common.core.controller.BaseController;
import com.slibra.common.core.domain.AjaxResult;
import com.slibra.common.enums.BusinessType;
import com.slibra.business.domain.TBucketRecord;
import com.slibra.business.service.ITBucketRecordService;
import com.slibra.common.utils.poi.ExcelUtil;
import com.slibra.common.core.page.TableDataInfo;

/**
 * 分流策略配置记录Controller
 * 
 * @author slibra
 * @date 2025-01-07
 */
@RestController
@RequestMapping("/business/bucketRecord")
public class TBucketRecordController extends BaseController
{
    @Autowired
    private ITBucketRecordService tBucketRecordService;

    /**
     * 查询分流策略配置记录列表
     */
    @PreAuthorize("@ss.hasPermi('business:record:list')")
    @GetMapping("/list")
    public TableDataInfo list(TBucketRecord tBucketRecord)
    {
        startPage();
        List<TBucketRecord> list = tBucketRecordService.selectTBucketRecordList(tBucketRecord);
        return getDataTable(list);
    }

    /**
     * 导出分流策略配置记录列表
     */
    @PreAuthorize("@ss.hasPermi('business:record:export')")
    @Log(title = "分流策略配置记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TBucketRecord tBucketRecord)
    {
        List<TBucketRecord> list = tBucketRecordService.selectTBucketRecordList(tBucketRecord);
        ExcelUtil<TBucketRecord> util = new ExcelUtil<TBucketRecord>(TBucketRecord.class);
        util.exportExcel(response, list, "分流策略配置记录数据");
    }

    /**
     * 获取分流策略配置记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:record:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(tBucketRecordService.selectTBucketRecordById(id));
    }

    /**
     * 新增分流策略配置记录
     */
    @PreAuthorize("@ss.hasPermi('business:record:add')")
    @Log(title = "分流策略配置记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TBucketRecord tBucketRecord)
    {
        tBucketRecord.setCreateBy(getUsername());
        return toAjax(tBucketRecordService.insertTBucketRecord(tBucketRecord));
    }

    /**
     * 修改分流策略配置记录
     */
    @PreAuthorize("@ss.hasPermi('business:record:edit')")
    @Log(title = "分流策略配置记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TBucketRecord tBucketRecord)
    {
        tBucketRecord.setUpdateBy(getUsername());
        return toAjax(tBucketRecordService.updateTBucketRecord(tBucketRecord));
    }

    /**
     * 删除分流策略配置记录
     */
    @PreAuthorize("@ss.hasPermi('business:record:remove')")
    @Log(title = "分流策略配置记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tBucketRecordService.deleteTBucketRecordByIds(ids));
    }


    /**
     * 修改分流策略配置
     */
    @PutMapping("/editConf")
    public AjaxResult editConf(@RequestBody TBucketRecord tBucketRecord)
    {
        return success(tBucketRecordService.editConf(tBucketRecord));
    }
}
