package com.slibra.web.controller.business;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.slibra.common.annotation.Log;
import com.slibra.common.core.controller.BaseController;
import com.slibra.common.core.domain.AjaxResult;
import com.slibra.common.enums.BusinessType;
import com.slibra.business.domain.TNeighborhoodBuilding;
import com.slibra.business.service.ITNeighborhoodBuildingService;
import com.slibra.common.utils.poi.ExcelUtil;
import com.slibra.common.core.page.TableDataInfo;

/**
 * 楼号信息Controller
 * 
 * @author slibra
 * @date 2024-11-05
 */
@RestController
@RequestMapping("/business/building")
public class TNeighborhoodBuildingController extends BaseController
{
    @Autowired
    private ITNeighborhoodBuildingService tNeighborhoodBuildingService;

    /**
     * 查询楼号信息列表
     */
//    @PreAuthorize("@ss.hasPermi('business:building:list')")
    @GetMapping("/list")
    public TableDataInfo list(TNeighborhoodBuilding tNeighborhoodBuilding)
    {
        startPage();
        List<TNeighborhoodBuilding> list = tNeighborhoodBuildingService.selectTNeighborhoodBuildingList(tNeighborhoodBuilding);
        return getDataTable(list);
    }

    /**
     * 导出楼号信息列表
     */
//    @PreAuthorize("@ss.hasPermi('business:building:export')")
//    @Log(title = "楼号信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TNeighborhoodBuilding tNeighborhoodBuilding)
    {
        List<TNeighborhoodBuilding> list = tNeighborhoodBuildingService.selectTNeighborhoodBuildingList(tNeighborhoodBuilding);
        ExcelUtil<TNeighborhoodBuilding> util = new ExcelUtil<TNeighborhoodBuilding>(TNeighborhoodBuilding.class);
        util.exportExcel(response, list, "楼号信息数据");
    }

    /**
     * 获取楼号信息详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:building:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(tNeighborhoodBuildingService.selectTNeighborhoodBuildingById(id));
    }

    /**
     * 新增楼号信息
     */
//    @PreAuthorize("@ss.hasPermi('business:building:add')")
//    @Log(title = "楼号信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TNeighborhoodBuilding tNeighborhoodBuilding)
    {
        tNeighborhoodBuilding.setCreateBy(getUsername());
        return toAjax(tNeighborhoodBuildingService.insertTNeighborhoodBuilding(tNeighborhoodBuilding));
    }

    /**
     * 修改楼号信息
     */
//    @PreAuthorize("@ss.hasPermi('business:building:edit')")
//    @Log(title = "楼号信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TNeighborhoodBuilding tNeighborhoodBuilding)
    {
        tNeighborhoodBuilding.setUpdateBy(getUsername());
        return toAjax(tNeighborhoodBuildingService.updateTNeighborhoodBuilding(tNeighborhoodBuilding));
    }

    /**
     * 删除楼号信息
     */
//    @PreAuthorize("@ss.hasPermi('business:building:remove')")
//    @Log(title = "楼号信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tNeighborhoodBuildingService.deleteTNeighborhoodBuildingByIds(ids));
    }
}
