package com.slibra.web.controller.business;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson2.JSON;
import com.slibra.business.domain.TNeighborhood;
import com.slibra.business.mapper.TNeighborhoodMapper;
import com.slibra.business.mapper.TPumpingStationMapper;
import com.slibra.business.req.NeighbourhoodNumberAndAddFlagReq;
import com.slibra.business.res.NeighbourhoodNumberAndAddFlag;
import com.slibra.business.service.ITNeighborhoodService;
import com.slibra.common.core.domain.R;
import lombok.extern.slf4j.Slf4j;
import org.apache.tomcat.Jar;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.slibra.common.annotation.Log;
import com.slibra.common.core.controller.BaseController;
import com.slibra.common.core.domain.AjaxResult;
import com.slibra.common.enums.BusinessType;
import com.slibra.business.domain.TPumpingStation;
import com.slibra.business.service.ITPumpingStationService;
import com.slibra.common.utils.poi.ExcelUtil;
import com.slibra.common.core.page.TableDataInfo;

/**
 * 泵站Controller
 * 
 * @author slibra
 * @date 2024-11-05
 */
@Slf4j
@RestController
@RequestMapping("/business/station")
public class TPumpingStationController extends BaseController
{
    @Autowired
    private ITPumpingStationService tPumpingStationService;

    @Autowired
    private TNeighborhoodMapper tNeighborhoodMapper;

    @Autowired
    private TPumpingStationMapper tPumpingStationMapper;

    /**
     * 查询泵站列表
     */
//    @PreAuthorize("@ss.hasPermi('business:station:list')")
    @GetMapping("/list")
    public TableDataInfo list(TPumpingStation tPumpingStation)
    {
        startPage();
        List<TPumpingStation> list = tPumpingStationService.selectTPumpingStationList(tPumpingStation);
        return getDataTable(list);
    }

    /**
     * 导出泵站列表
     */
//    @PreAuthorize("@ss.hasPermi('business:station:export')")
//    @Log(title = "泵站", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TPumpingStation tPumpingStation)
    {
        List<TPumpingStation> list = tPumpingStationService.selectTPumpingStationList(tPumpingStation);
        ExcelUtil<TPumpingStation> util = new ExcelUtil<TPumpingStation>(TPumpingStation.class);
        util.exportExcel(response, list, "泵站数据");
    }

    /**
     * 获取泵站详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:station:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(tPumpingStationService.selectTPumpingStationById(id));
    }

    /**
     * 新增泵站
     */
//    @PreAuthorize("@ss.hasPermi('business:station:add')")
//    @Log(title = "泵站", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPumpingStation tPumpingStation)
    {
        tPumpingStation.setCreateBy(getUsername());
        return toAjax(tPumpingStationService.insertTPumpingStation(tPumpingStation));
    }

    /**
     * 修改泵站
     */
//    @PreAuthorize("@ss.hasPermi('business:station:edit')")
//    @Log(title = "泵站", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPumpingStation tPumpingStation)
    {
        tPumpingStation.setUpdateBy(getUsername());
        return toAjax(tPumpingStationService.updateTPumpingStation(tPumpingStation));
    }

    /**
     * 删除泵站
     */
//    @PreAuthorize("@ss.hasPermi('business:station:remove')")
//    @Log(title = "泵站", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tPumpingStationService.deleteTPumpingStationByIds(ids));
    }

    /**
     * 获取所有的小区(简单信息 没有查询复杂数据)
     * 2024年11月19日17:31:38 逻辑调整：前端再把对应的泵站ID传过来，然后查询绑定的小区楼号相关的信息
     * @return
     */
    @GetMapping("/getSimpleNeighbourhoodList")
    public R<List<TNeighborhood>> getSimpleNeighbourhoodList(TNeighborhood tNeighborhood){
        log.info("进入了 获取小区列表基本信息 接口");
        return R.ok(this.tPumpingStationService.getSimpleNeighbourhoodList(tNeighborhood));
    }


    /**
     * 获取所有的泵站(简单信息 没有查询复杂数据)
     * @return
     */
    @GetMapping("/getSimplePumpingStationList")
    public R<List<TPumpingStation>> getSimplePumpingStationList(TPumpingStation tPumpingStation){
        log.info("进入了 获取所有的泵站基本信息 接口");
        return R.ok(this.tPumpingStationMapper.selectTPumpingStationList(tPumpingStation));
    }


    /**
     * 通过泵站ID和小区ID查询所有的楼号，并且是否已经添加到该泵站下了
     * @return
     */
    @GetMapping("/getBuildingsAndFlagByID2")
    public R<List<NeighbourhoodNumberAndAddFlag>> getBuildingsAndFlagByID2(NeighbourhoodNumberAndAddFlagReq req){
        log.info("进入了 过泵站ID和小区ID查询所有的楼号，并且是否已经添加到该泵站下了 接口，请求参数为{}", JSON.toJSONString(req));
        return R.ok(this.tPumpingStationService.getBuildingsAndFlagByID2(req));
    }


    /**
     * 给泵站关联小区的楼号
     * @return
     */
    @PostMapping("/addPumpingStationAndNeighbourhoodBuildings")
    public R<String> addPumpingStationAndNeighbourhoodBuildings(@RequestBody List<TNeighborhood> req){
        log.info("进入了 给泵站关联小区的楼号 接口，请求参数为{}", JSON.toJSONString(req));
        return R.ok(this.tPumpingStationService.addPumpingStationAndNeighbourhoodBuildings(req));
    }
}
