package com.slibra.web.controller.business;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.slibra.common.annotation.Log;
import com.slibra.common.core.controller.BaseController;
import com.slibra.common.core.domain.AjaxResult;
import com.slibra.common.enums.BusinessType;
import com.slibra.business.domain.TWhitelist;
import com.slibra.business.service.ITWhitelistService;
import com.slibra.common.utils.poi.ExcelUtil;
import com.slibra.common.core.page.TableDataInfo;

/**
 * 白名单配置Controller
 * 
 * @author slibra
 * @date 2024-11-05
 */
@RestController
@RequestMapping("/business/whitelist")
public class TWhitelistController extends BaseController
{
    @Autowired
    private ITWhitelistService tWhitelistService;

    /**
     * 查询白名单配置列表
     */
//    @PreAuthorize("@ss.hasPermi('business:whitelist:list')")
    @GetMapping("/list")
    public TableDataInfo list(TWhitelist tWhitelist)
    {
        startPage();
        List<TWhitelist> list = tWhitelistService.selectTWhitelistList(tWhitelist);
        return getDataTable(list);
    }

    /**
     * 导出白名单配置列表
     */
//    @PreAuthorize("@ss.hasPermi('business:whitelist:export')")
//    @Log(title = "白名单配置", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TWhitelist tWhitelist)
    {
        List<TWhitelist> list = tWhitelistService.selectTWhitelistList(tWhitelist);
        ExcelUtil<TWhitelist> util = new ExcelUtil<TWhitelist>(TWhitelist.class);
        util.exportExcel(response, list, "白名单配置数据");
    }

    /**
     * 获取白名单配置详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:whitelist:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(tWhitelistService.selectTWhitelistById(id));
    }

    /**
     * 新增白名单配置
     */
//    @PreAuthorize("@ss.hasPermi('business:whitelist:add')")
//    @Log(title = "白名单配置", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TWhitelist tWhitelist)
    {
        tWhitelist.setCreateBy(getUsername());
        return toAjax(tWhitelistService.insertTWhitelist(tWhitelist));
    }

    /**
     * 修改白名单配置
     */
//    @PreAuthorize("@ss.hasPermi('business:whitelist:edit')")
//    @Log(title = "白名单配置", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TWhitelist tWhitelist)
    {
        tWhitelist.setUpdateBy(getUsername());
        return toAjax(tWhitelistService.updateTWhitelist(tWhitelist));
    }

    /**
     * 删除白名单配置
     */
//    @PreAuthorize("@ss.hasPermi('business:whitelist:remove')")
//    @Log(title = "白名单配置", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tWhitelistService.deleteTWhitelistByIds(ids));
    }
}
