package com.slibra.business.domain;

import java.util.Date;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import com.slibra.common.annotation.Excel;
import com.slibra.common.core.domain.BaseEntity;

/**
 * 通话记录对象 t_call_record
 * 
 * @author slibra
 * @date 2024-11-11
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class TCallRecord extends BaseEntity
{
    private static final long serialVersionUID = 1L;

    /** 主键 */
    private Long id;

    /** sessionId */
    @Excel(name = "sessionId")
    private String sessionId;

    /** 呼入分类（0白名单 1AI服务 2传统服务） */
    @Excel(name = "呼入分类", readConverterExp = "0=白名单,1=AI服务,2=传统服务")
    private Long type;

    /** 客服ID */
    @Excel(name = "客服ID")
    private Long userId;

    /** 客服名字 */
    @Excel(name = "客服名字")
    private String userName;

    /** 服务类型（0人工坐席 1机器人坐席 2机器人转人工） */
    @Excel(name = "服务类型", readConverterExp = "0=人工坐席,1=机器人坐席,2=机器人转人工")
    private Long serviceCategory;

    /** 通话发起时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "通话发起时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    private Date timeBegin;

    /** 通话结束时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "通话结束时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    private Date timeEnd;

    /** 通话时长（暂时按字符串接收） */
    @Excel(name = "通话时长", readConverterExp = "暂=时按字符串接收")
    private String times;

    /** 通话类型（0呼入 1呼出） */
    @Excel(name = "通话类型", readConverterExp = "0=呼入,1=呼出")
    private Long category;

    /** 通话状态(0未接听 1已接通) */
    @Excel(name = "通话状态(0未接听 1已接通)")
    private Long status;

    /** 电话号码 */
    @Excel(name = "电话号码")
    private String phone;

    /** 业务类型（创个返回字符串） */
    @Excel(name = "业务类型", readConverterExp = "创=个返回字符串")
    private String bussinessType;

    //存储语音的URL
    private String url;

    /** 是否已转录音（0否 1是） */
    @Excel(name = "是否已转录音", readConverterExp = "0=否,1=是")
    private Long hasParsed;

    /** 通话录音内容 */
    @Excel(name = "通话录音内容")
    private String parsedVoiceContent;

    /** 删除标志（0代表存在 2代表删除） */
    private Long delFlag;

    /** 乐观锁 */
    @Excel(name = "乐观锁")
    private Long revision;

    //2025年01月07日16:46:51 新增的数据库字段
    /** 客服ID */
    @Excel(name = "转接客服id")
    private Long transferUserId;

    /** 客服名字 */
    @Excel(name = "转接客服名字")
    private String transferUserName;

    //2024年11月18日10:45:25 新增的返回的字段[当前通话记录]
    private String currentCallRecords;

    //2024年11月18日16:25:26 新增一些统计相关的信息 【户号水费小区名称  拨打次数  历史通话记录  停水信息】
    private int counts;//拨打次数

    private List<TUserInfo> userInfos;//户号水费小区名称

    private List<TCallRecord> callRecords;//历史通话记录

    private List<TCutOffWater> cutOffWaters;//正在进行的停水信息

    //2025年01月06日13:42:28 新增的请求参数
    //通话开始时间-开始
    private String timeBeginReq;

    //通话开始时间-截止
    private String timeEndReq;


}
