package com.slibra.business.domain;

import java.math.BigDecimal;
import java.util.Date;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import com.slibra.common.annotation.Excel;
import com.slibra.common.core.domain.BaseEntity;

/**
 * 用户信息对象 t_user_info
 * 
 * @author slibra
 * @date 2024-11-08
 */
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class TUserInfo extends BaseEntity
{
    private static final long serialVersionUID = 1L;

    /** 主键 */
    private Long id;

    /** 用户编号 */
    @Excel(name = "用户编号")
    private String userNo;

    /** 户号 */
    @Excel(name = "户号")
    private String cardNo;

    /** 表号 */
    @Excel(name = "表号")
    private String ammeterNo;

    /** 用户名称 */
    @Excel(name = "用户名称")
    private String name;

    /** 移动电话（来电号码） */
    @Excel(name = "移动电话", readConverterExp = "来=电号码")
    private String phone;

    /** 省[冗余字段] */
    @JsonIgnore
    @Excel(name = "省[冗余字段]")
    private String province;

    /** 市[冗余字段] */
    @JsonIgnore
    @Excel(name = "市[冗余字段]")
    private String city;

    /** 区县[冗余字段] */
    @JsonIgnore
    @Excel(name = "区县[冗余字段]")
    private String country;

    /** 水表地址 */
    @Excel(name = "水表地址")
    private String pumpingStationAddress;

    /** 地区 */
    @Excel(name = "地区")
    private String area;

    /** 街道 */
    @Excel(name = "街道")
    private String street;

    /** 小区名称 */
    @Excel(name = "小区名称")
    private String neighbourhoodName;

    /** 楼号 */
    @Excel(name = "楼号")
    private String buildingNo;

    /** 门牌号 */
    @Excel(name = "门牌号")
    private String doorNo;

    /** 最新抄表日期 */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @Excel(name = "最新抄表日期", width = 30, dateFormat = "yyyy-MM-dd")
    private Date statisticsTime;

    /** 未扣费总额[冗余字段] */
    @JsonIgnore
    @Excel(name = "未扣费总额[冗余字段]")
    private BigDecimal unpaidFees;

    /** 账户余额[冗余字段] */
    @JsonIgnore
    @Excel(name = "账户余额[冗余字段]")
    private BigDecimal balanceFees;

    /** 欠费金额 */
    @Excel(name = "欠费金额")
    private BigDecimal waterFees;

    /** 抄表表数 */
    @Excel(name = "抄表表数")
    private BigDecimal meterAmount;

    /** 抄表员姓名 */
    @Excel(name = "抄表员姓名")
    private String meterReader;

    /** 抄表员电话 */
    @Excel(name = "抄表员电话")
    private String meterReaderPhone;

    /** 删除标志（0代表存在 2代表删除） */
    @JsonIgnore
    private Long delFlag;

    /** 乐观锁 */
    @JsonIgnore
    @Excel(name = "乐观锁")
    private Long revision;

}
