package com.slibra.business.mapper;

import java.util.HashMap;
import java.util.List;

import com.slibra.business.domain.TCallRecord;
import com.slibra.business.res.CallUserCountInfo;
import com.slibra.business.res.UserTypeResp;
import org.apache.ibatis.annotations.Param;

/**
 * 通话记录Mapper接口
 * 
 * @author slibra
 * @date 2024-11-11
 */
public interface TCallRecordMapper 
{
    /**
     * 查询通话记录
     * 
     * @param id 通话记录主键
     * @return 通话记录
     */
    public TCallRecord selectTCallRecordById(Long id);

    /**
     * 查询通话记录列表
     * 
     * @param tCallRecord 通话记录
     * @return 通话记录集合
     */
    public List<TCallRecord> selectTCallRecordList(TCallRecord tCallRecord);

    /**
     * 新增通话记录
     * 
     * @param tCallRecord 通话记录
     * @return 结果
     */
    public int insertTCallRecord(TCallRecord tCallRecord);

    /**
     * 修改通话记录
     * 
     * @param tCallRecord 通话记录
     * @return 结果
     */
    public int updateTCallRecord(TCallRecord tCallRecord);

    /**
     * 删除通话记录
     * 
     * @param id 通话记录主键
     * @return 结果
     */
    public int deleteTCallRecordById(Long id);

    /**
     * 批量删除通话记录
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTCallRecordByIds(Long[] ids);

    String getBusinessTypeBySessionId(String sessionId);

    String getContentsBySessionId(String sessionId);

    List<UserTypeResp> getAgentList();

    int getCallCountsByPhone(String phone);

    List<TCallRecord> selectHistoriesByPhone(@Param("phone") String phone, @Param("id") Long id);

    int getTotalByType(Integer type);

    int getTotalByIsAi(int businessType);

    int getSpecialCount();

    int getTransferCount(@Param("userId") String userId, @Param("date") String date);

    int getPersonCountByState(Integer state);

    int getPersonCountRecordIsRobot(int type);

    long getPersonTotalRecordIsRobot(@Param("type") int type, @Param("userId") String userId);

    List<CallUserCountInfo> selectAllUsersByType(int type);

    Integer getStatusByUserId(String userId);

    int getCountByTypeAndDate(@Param("category") int category, @Param("date") String date, @Param("userId") String userId);

    int getPersonCountByType(int type);

    List<CallUserCountInfo> selectAllPersonUserIds();
}
