package com.slibra.business.service.impl;

import java.util.List;
import com.slibra.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.slibra.business.mapper.CBucketMapper;
import com.slibra.business.domain.CBucket;
import com.slibra.business.service.ICBucketService;

/**
 * 流量分桶Service业务层处理
 * 
 * @author slibra
 * @date 2025-01-06
 */
@Service
public class CBucketServiceImpl implements ICBucketService 
{
    @Autowired
    private CBucketMapper cBucketMapper;

    /**
     * 查询流量分桶
     * 
     * @param id 流量分桶主键
     * @return 流量分桶
     */
    @Override
    public CBucket selectCBucketById(String id)
    {
        return cBucketMapper.selectCBucketById(id);
    }

    /**
     * 查询流量分桶列表
     * 
     * @param cBucket 流量分桶
     * @return 流量分桶
     */
    @Override
    public List<CBucket> selectCBucketList(CBucket cBucket)
    {
        return cBucketMapper.selectCBucketList(cBucket);
    }

    /**
     * 新增流量分桶
     * 
     * @param cBucket 流量分桶
     * @return 结果
     */
    @Override
    public int insertCBucket(CBucket cBucket)
    {
        cBucket.setCreateTime(DateUtils.getNowDate());
        return cBucketMapper.insertCBucket(cBucket);
    }

    /**
     * 修改流量分桶
     * 
     * @param cBucket 流量分桶
     * @return 结果
     */
    @Override
    public int updateCBucket(CBucket cBucket)
    {
        cBucket.setUpdateTime(DateUtils.getNowDate());
        return cBucketMapper.updateCBucket(cBucket);
    }

    /**
     * 批量删除流量分桶
     * 
     * @param ids 需要删除的流量分桶主键
     * @return 结果
     */
    @Override
    public int deleteCBucketByIds(String[] ids)
    {
        return cBucketMapper.deleteCBucketByIds(ids);
    }

    /**
     * 删除流量分桶信息
     * 
     * @param id 流量分桶主键
     * @return 结果
     */
    @Override
    public int deleteCBucketById(String id)
    {
        return cBucketMapper.deleteCBucketById(id);
    }
}
