package com.slibra.business.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Objects;

import com.slibra.business.domain.CBucket;
import com.slibra.business.mapper.CBucketMapper;
import com.slibra.common.exception.ServiceException;
import com.slibra.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.slibra.business.mapper.TBucketRecordMapper;
import com.slibra.business.domain.TBucketRecord;
import com.slibra.business.service.ITBucketRecordService;
import org.springframework.transaction.annotation.Transactional;

import static com.slibra.common.utils.SecurityUtils.getUsername;

/**
 * 分流策略配置记录Service业务层处理
 * 
 * @author slibra
 * @date 2025-01-07
 */
@Service
public class TBucketRecordServiceImpl implements ITBucketRecordService 
{
    @Autowired
    private TBucketRecordMapper tBucketRecordMapper;

    @Autowired
    private CBucketMapper cBucketMapper;

    /**
     * 查询分流策略配置记录
     * 
     * @param id 分流策略配置记录主键
     * @return 分流策略配置记录
     */
    @Override
    public TBucketRecord selectTBucketRecordById(Long id)
    {
        return tBucketRecordMapper.selectTBucketRecordById(id);
    }

    /**
     * 查询分流策略配置记录列表
     * 
     * @param tBucketRecord 分流策略配置记录
     * @return 分流策略配置记录
     */
    @Override
    public List<TBucketRecord> selectTBucketRecordList(TBucketRecord tBucketRecord)
    {
        return tBucketRecordMapper.selectTBucketRecordList(tBucketRecord);
    }

    /**
     * 新增分流策略配置记录
     * 
     * @param tBucketRecord 分流策略配置记录
     * @return 结果
     */
    @Override
    public int insertTBucketRecord(TBucketRecord tBucketRecord)
    {
        tBucketRecord.setCreateTime(DateUtils.getNowDate());
        return tBucketRecordMapper.insertTBucketRecord(tBucketRecord);
    }

    /**
     * 修改分流策略配置记录
     * 
     * @param tBucketRecord 分流策略配置记录
     * @return 结果
     */
    @Override
    public int updateTBucketRecord(TBucketRecord tBucketRecord)
    {
        tBucketRecord.setUpdateTime(DateUtils.getNowDate());
        return tBucketRecordMapper.updateTBucketRecord(tBucketRecord);
    }

    /**
     * 批量删除分流策略配置记录
     * 
     * @param ids 需要删除的分流策略配置记录主键
     * @return 结果
     */
    @Override
    public int deleteTBucketRecordByIds(Long[] ids)
    {
        return tBucketRecordMapper.deleteTBucketRecordByIds(ids);
    }

    /**
     * 删除分流策略配置记录信息
     * 
     * @param id 分流策略配置记录主键
     * @return 结果
     */
    @Override
    public int deleteTBucketRecordById(Long id)
    {
        return tBucketRecordMapper.deleteTBucketRecordById(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String editConf(TBucketRecord tBucketRecord) {
        Integer traditionPercent = tBucketRecord.getTraditionPercent();
//        Integer aiPercent = tBucketRecord.getAiPercent();
//        if(Objects.isNull(traditionPercent) || Objects.isNull(aiPercent))
        if(Objects.isNull(traditionPercent))
            throw new ServiceException("传统服务占比配置不能为空");
        Date nowDate = DateUtils.getNowDate();
        tBucketRecord.setCreateBy(getUsername());
        tBucketRecord.setCreateTime(nowDate);
        this.tBucketRecordMapper.insertTBucketRecord(tBucketRecord);
        //更新配置表
        CBucket cBucketTradition = this.cBucketMapper.selectCBucketById(String.valueOf(1));
        CBucket cBucketAI = this.cBucketMapper.selectCBucketById(String.valueOf(2));
        if(!Objects.isNull(cBucketTradition) && !Objects.isNull(cBucketAI)){
            cBucketTradition.setLower(0);
            cBucketTradition.setUpper(traditionPercent);
            cBucketTradition.setUpdateTime(nowDate);
            this.cBucketMapper.updateCBucket(cBucketTradition);
            cBucketAI.setLower(traditionPercent);
            cBucketAI.setUpper(100);
            cBucketAI.setUpdateTime(nowDate);
            this.cBucketMapper.updateCBucket(cBucketAI);
        }
        return "修改成功";
    }
}
