package com.slibra.business.service.impl;

import java.util.List;
import com.slibra.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.slibra.business.mapper.TUserFeeInfoMapper;
import com.slibra.business.domain.TUserFeeInfo;
import com.slibra.business.service.ITUserFeeInfoService;

/**
 * 用户水费明细Service业务层处理
 * 
 * @author slibra
 * @date 2024-11-05
 */
@Service
public class TUserFeeInfoServiceImpl implements ITUserFeeInfoService 
{
    @Autowired
    private TUserFeeInfoMapper tUserFeeInfoMapper;

    /**
     * 查询用户水费明细
     * 
     * @param id 用户水费明细主键
     * @return 用户水费明细
     */
    @Override
    public TUserFeeInfo selectTUserFeeInfoById(Long id)
    {
        return tUserFeeInfoMapper.selectTUserFeeInfoById(id);
    }

    /**
     * 查询用户水费明细列表
     * 
     * @param tUserFeeInfo 用户水费明细
     * @return 用户水费明细
     */
    @Override
    public List<TUserFeeInfo> selectTUserFeeInfoList(TUserFeeInfo tUserFeeInfo)
    {
        return tUserFeeInfoMapper.selectTUserFeeInfoList(tUserFeeInfo);
    }

    /**
     * 新增用户水费明细
     * 
     * @param tUserFeeInfo 用户水费明细
     * @return 结果
     */
    @Override
    public int insertTUserFeeInfo(TUserFeeInfo tUserFeeInfo)
    {
        tUserFeeInfo.setCreateTime(DateUtils.getNowDate());
        return tUserFeeInfoMapper.insertTUserFeeInfo(tUserFeeInfo);
    }

    /**
     * 修改用户水费明细
     * 
     * @param tUserFeeInfo 用户水费明细
     * @return 结果
     */
    @Override
    public int updateTUserFeeInfo(TUserFeeInfo tUserFeeInfo)
    {
        tUserFeeInfo.setUpdateTime(DateUtils.getNowDate());
        return tUserFeeInfoMapper.updateTUserFeeInfo(tUserFeeInfo);
    }

    /**
     * 批量删除用户水费明细
     * 
     * @param ids 需要删除的用户水费明细主键
     * @return 结果
     */
    @Override
    public int deleteTUserFeeInfoByIds(Long[] ids)
    {
        return tUserFeeInfoMapper.deleteTUserFeeInfoByIds(ids);
    }

    /**
     * 删除用户水费明细信息
     * 
     * @param id 用户水费明细主键
     * @return 结果
     */
    @Override
    public int deleteTUserFeeInfoById(Long id)
    {
        return tUserFeeInfoMapper.deleteTUserFeeInfoById(id);
    }
}
