package com.slibra.business.service.impl;

import java.util.List;

import com.slibra.common.exception.ServiceException;
import com.slibra.common.utils.DateUtils;
import com.slibra.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.slibra.business.mapper.TWhitelistMapper;
import com.slibra.business.domain.TWhitelist;
import com.slibra.business.service.ITWhitelistService;
import org.springframework.util.CollectionUtils;

/**
 * 白名单配置Service业务层处理
 * 
 * @author slibra
 * @date 2024-11-05
 */
@Service
public class TWhitelistServiceImpl implements ITWhitelistService 
{
    @Autowired
    private TWhitelistMapper tWhitelistMapper;

    /**
     * 查询白名单配置
     * 
     * @param id 白名单配置主键
     * @return 白名单配置
     */
    @Override
    public TWhitelist selectTWhitelistById(Long id)
    {
        return tWhitelistMapper.selectTWhitelistById(id);
    }

    /**
     * 查询白名单配置列表
     * 
     * @param tWhitelist 白名单配置
     * @return 白名单配置
     */
    @Override
    public List<TWhitelist> selectTWhitelistList(TWhitelist tWhitelist)
    {
        return tWhitelistMapper.selectTWhitelistList(tWhitelist);
    }

    /**
     * 新增白名单配置
     * 
     * @param tWhitelist 白名单配置
     * @return 结果
     */
    @Override
    public int insertTWhitelist(TWhitelist tWhitelist)
    {
        tWhitelist.setCreateTime(DateUtils.getNowDate());
        //2024年11月12日10:48:13 校验是否已经存在
        if(!CollectionUtils.isEmpty(this.tWhitelistMapper.selectTWhitelistList(TWhitelist.builder().phone(tWhitelist.getPhone()).build())))
            throw new ServiceException("该电话号码已经存在白名单中了，请勿重复添加！");
        return tWhitelistMapper.insertTWhitelist(tWhitelist);
    }

    /**
     * 修改白名单配置
     * 
     * @param tWhitelist 白名单配置
     * @return 结果
     */
    @Override
    public int updateTWhitelist(TWhitelist tWhitelist)
    {
        tWhitelist.setUpdateTime(DateUtils.getNowDate());
        //2024年11月12日10:48:13 校验是否已经存在
        if(StringUtils.isNotBlank(this.tWhitelistMapper.getUniquePhoneButNoThis(tWhitelist)))
            throw new ServiceException("该电话号码已经存在白名单中了，请勿重复添加！");
        return tWhitelistMapper.updateTWhitelist(tWhitelist);
    }

    /**
     * 批量删除白名单配置
     * 
     * @param ids 需要删除的白名单配置主键
     * @return 结果
     */
    @Override
    public int deleteTWhitelistByIds(Long[] ids)
    {
        return tWhitelistMapper.deleteTWhitelistByIds(ids);
    }

    /**
     * 删除白名单配置信息
     * 
     * @param id 白名单配置主键
     * @return 结果
     */
    @Override
    public int deleteTWhitelistById(Long id)
    {
        return tWhitelistMapper.deleteTWhitelistById(id);
    }
}
